@@ -41,11 +41,14 @@
 //! - [Wildcard Example](https://github.com/w-henderson/Humphrey/tree/master/examples/wildcard): demonstrates a wildcard route
 //! - [Static Content Example](https://github.com/w-henderson/Humphrey/tree/master/examples/static-content): demonstrates the built-in static content handlers
 
+#![warn(missing_docs)]
+
 pub mod app;
 pub mod handlers;
 pub mod http;
 pub mod krauss;
 pub mod route;
+pub mod stream;
 pub mod thread;
 
 #[cfg(test)]
